package com.portio

import android.app.Application
import com.portio.data.repository.UserPreferencesRepository
import android.content.Context
import androidx.activity.ComponentActivity
import androidx.compose.runtime.Composable
import androidx.compose.runtime.collectAsState
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.datastore.preferences.preferencesDataStore
import androidx.lifecycle.viewmodel.compose.viewModel
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import com.portio.data.repository.LocalPlateImageDataSource
import com.portio.data.repository.LocalPlateImageRepository
import com.portio.data.repository.LocalUserAvatarDataSource
import com.portio.data.repository.LocalUserAvatarRepository
import com.portio.ui.screens.home.HomeScreen
import com.portio.ui.screens.platemanager.EditPlateScreen
import com.portio.ui.screens.platemanager.PlateManagerScreen
import com.portio.ui.screens.usermanager.UserManagerScreen

private const val USER_PREFERENCES_NAME = "user_preferences"
private val Context.dataStore by preferencesDataStore(
    name = USER_PREFERENCES_NAME
)

/**
 * com.portio.PortioApp is the entry point to the Portio app.
 * In future steps, should manage the window size
 */
@Composable
fun PortioApp(modifier: Modifier = Modifier) {
    val context = LocalContext.current

    val activity = context as ComponentActivity
    val viewModel: PortioViewModel = viewModel(
        factory = PortioViewModelFactory(
            application = context.applicationContext as Application,
            userPreferencesRepository = UserPreferencesRepository(dataStore = activity.dataStore),
            localPlateImageRepository = LocalPlateImageRepository(
                localDataSource = LocalPlateImageDataSource(context)
            ),
            localUserAvatarRepository = LocalUserAvatarRepository(
                localDataSource = LocalUserAvatarDataSource(context)
            )
        )
    )

    val uiState = viewModel.uiState.collectAsState().value
    val navController = rememberNavController()

    NavHost(
        navController = navController,
        startDestination = Home.route,
        modifier = Modifier
    ) {
        composable(route = Home.route) {
            HomeScreen(
                modifier = modifier,
                uiState = uiState,
                onWeightChanged = viewModel::setTotalWeight,
                onMealCountChanged = viewModel::setMealCount,
                onActivePlateChanged = viewModel::setActivePlate,
                onManageUserClick = { navController.navigate(UserManager.route) },
                onManagePlateClick = { navController.navigate(PlateManager.route) }
            )
        }
        composable(route = UserManager.route) {
            UserManagerScreen(
                modifier = modifier,
                userList = uiState.userList.map { it.first },
                ratioMode = uiState.ratioMode,
                onNavigationClicked = { navController.popBackStack() },
                onUserRemoved = { user ->
                    viewModel.removeUser(user)
                },
                onAddOrEditUser = viewModel::editUser,
                onChangeRatioMode = viewModel::changeRatioMode
            )
        }
        composable(route = PlateManager.route) {
            PlateManagerScreen(
                modifier = modifier,
                plateList = uiState.plateList,
                onMove = viewModel::reorderPlates,
                onNavigationClicked = { navController.popBackStack() },
                onPlateRemoved = viewModel::removePlate,
                onEditPlateClick = { plate ->
                    navController.navigate("${EditPlate.route}/${plate.id}")
                },
                onAddPlateClick = {
                    navController.navigate("${EditPlate.route}/")
                },
            )
        }
        composable(route = "${EditPlate.route}/{plateId}") { backStackEntry ->
            val plateId = backStackEntry.arguments?.getString("plateId") ?: return@composable
            val plate = uiState.plateList.find { it.id.toString() == plateId }

            EditPlateScreen(
                modifier = modifier,
                plate = plate,
                onNavigationClicked = { navController.popBackStack() },
                onPlateEdited = {
                    viewModel.editPlate(it)
                    navController.popBackStack()
                },
            )
        }
    }
}

