package com.portio

import android.app.Application
import android.net.Uri
import com.portio.data.repository.UserPreferencesRepository
import android.util.Log
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.viewModelScope
import com.portio.data.Plate
import com.portio.data.User
import com.portio.data.repository.LocalPlateImageRepository
import com.portio.data.repository.LocalUserAvatarRepository
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.update
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import kotlin.collections.sumOf
import kotlin.math.min

class PortioViewModel(
    application: Application,
    private val userPreferencesRepository: UserPreferencesRepository,
    private val localPlateImageRepository: LocalPlateImageRepository,
    private val localUserAvatarRepository: LocalUserAvatarRepository
) : AndroidViewModel(application) {

    // Private mutable state for user data
    private val _uiState = MutableStateFlow(UiState())

    // Public state to observe user data
    val uiState: StateFlow<UiState> = _uiState

    private fun UiState.changeRatioMode(ratioMode: RatioMode) =
        copy(ratioMode = ratioMode, userList = userList.map { (user, _) ->
            val u = user.copy(
                desiredRatio = when (ratioMode) {
                    RatioMode.EQUALLY, RatioMode.AMOUNT -> -1
                    RatioMode.PARTS -> 1
                }
            )
            Pair(u, 0f)
        }).updatePercentage().updateWeight()

    private fun UiState.setMealCount(mealCount: Int) = copy(mealCount = mealCount)
        .updateWeight()

    private fun UiState.setTotalWeight(totalWeight: Float) = copy(totalWeight = totalWeight)
        .updateWeight()

    private fun UiState.addPlate(plate: Plate) = copy(plateList = plateList + plate)

    private fun UiState.removePlate(plate: Plate) = copy(plateList = plateList - plate)

    private fun UiState.editPlate(plate: Plate) = copy(
        plateList = plateList.map { p ->
            if (p.id == plate.id) plate else p
        }
    )

    private fun UiState.setActivePlate(plate: Plate) = copy(activePlate = plate)
        .updateWeight()

    private fun UiState.containsUser(user: User): Boolean = userList.any { (u, _) -> u == user }

    private fun UiState.containsUser(name: String): Boolean =
        userList.any { (u, _) -> u.name == name }

    private fun UiState.addUser(user: User) = copy(
        userList = userList + Pair(user, 0f)
    ).updatePercentage().updateWeight()

    private fun UiState.removeUser(user: User) = copy(
        userList = userList.filter { (u, _) -> u != user }
    ).updatePercentage().updateWeight()

    private fun UiState.editUser(user: User) = copy(
        userList = userList.map { (u, _) ->
            val updatedUser = if (u.id == user.id)
                u.copy(
                    desiredRatio = user.desiredRatio,
                    name = user.name,
                    imageUri = user.imageUri
                ) else u
            Pair(updatedUser, 0f)
        }
    ).updatePercentage().updateWeight()

//    private fun UiState.editUserDesiredRatio(user: User, desiredRatio: Int) = copy( // Todo: Implement
//        userList = userList.map { (u, bitmap, _) ->
//            val updatedUser = if (u.id == user.id) u.copy(desiredRatio = desiredRatio) else u
//            Triple(updatedUser, bitmap, 0f)
//        }
//    ).updatePercentage().updateWeight()

//    private fun UiState.editUserName(user: User, name: String) = copy( // Todo: Implement
//        userList = userList.map { (u, bitmap, _) ->
//            val updatedUser = if (u.id == user.id) u.copy(name = name) else u
//            Triple(updatedUser, bitmap, 0f)
//        }
//    ) // No need to update percentage and weight (only name)

    private fun UiState.updatePercentage() = copy( // Basic operation.
        userList = userList.map { (user, percentage) ->
            val updatedUser = user.copy(effectiveRatio = when (ratioMode) {
                RatioMode.EQUALLY -> 100 / userList.size // Simply divide by the number of users
                RatioMode.PARTS -> {
                    val totalParts = userList.sumOf { (u, _) -> u.desiredRatio }
                    100 * user.desiredRatio / totalParts
                }

                RatioMode.AMOUNT -> { // Calculate the percentage based on the desired ratio
                    if (user.desiredRatio != -1) user.desiredRatio
                    else {
                        // If the desired ratio is not set (-1),
                        // calculate the percentage based on the remaining users.
                        val totalDesired = userList
                            .sumOf { (u, _) -> if (u.desiredRatio != -1) u.desiredRatio else 0 }
                        val rest = 100 - min(totalDesired, 100)
                        val userWithValueUnset = userList
                            .filter { (u, _) -> u.desiredRatio == -1 }.size
                        rest / userWithValueUnset
                    }
                }
            }
            )
            Pair(updatedUser, percentage)
        }
    )

    private fun UiState.updateWeight() = copy( // Basic operation.
        userList = userList.map { (user, _) ->
            val newWeight = calculateWeight(
                totalWeight = totalWeight,
                plateWeight = activePlate?.weight ?: 0f,
                userPercentage = user.effectiveRatio,
                mealCount = mealCount
            )
            Pair(user, newWeight)
        }
    )

    private fun defaultPlates() = listOf(
        R.drawable.default_plate_1,
        R.drawable.default_plate_2,
        R.drawable.default_plate_3
    ).mapIndexed { index, drawable ->
        val context = getApplication<Application>().applicationContext
        Plate(
            name = context.getString(R.string.default_plate, index.toString()),
            weight = 250f * (index + 1),
            imageUri = Uri.parse("android.resource://${context.packageName}/$drawable")
        )
    }

    private fun defaultUsers(): List<User> {
        val context = getApplication<Application>().applicationContext
        return listOf(
            User(
                name = context.getString(R.string.default_user_1),
                imageUri = Uri.parse("android.resource://${context.packageName}/${R.drawable.default_user_rachel}")
            ),
            User(
                name = context.getString(R.string.default_user_2),
                imageUri = Uri.parse("android.resource://${context.packageName}/${R.drawable.default_user_michael}")
            )
        )
    }

    init {
        viewModelScope.launch {
            _uiState.update {
                userPreferencesRepository.fetchInitialPreferences().let { preferences ->
                    val plates = preferences.plates
                        ?: defaultPlates() // Show-room the app with default plates
                            .apply {
                                userPreferencesRepository.setActivePlate(this.first())
                                userPreferencesRepository.savePlates(this)
                            }
                    val users = preferences.users
                        ?: defaultUsers() // Show-room the app with default users
                            .onEach { userPreferencesRepository.addUser(it) }
                    UiState(
                        plateList = plates,
                        mealCount = preferences.mealCount,
                        userList = users.map { user -> Pair(user, 0f) },
                        activePlate = preferences.activePlate ?: plates.firstOrNull(),
                        totalWeight = preferences.totalWeight,
                        ratioMode = preferences.ratioMode
                    ).updatePercentage().updateWeight()
                }
            }
            // Search for users with null imageUri and fetch their images.
            // Can happen when the app is closed during image processing.
            uiState.value.userList.map { it.first }.forEach { user ->
                if (user.imageUri == null) {
                    Log.w(TAG, "User ($user) image is null")
                    viewModelScope.launch(Dispatchers.IO) {
                        val uri = localUserAvatarRepository.fetchAndSaveUserImage(user.name)
                        if (uri != null) {
                            val updatedUser = user.copy(imageUri = uri)
                            userPreferencesRepository.editUser(updatedUser)
                            withContext(Dispatchers.Main) {
                                _uiState.update { it.editUser(updatedUser) }
                            }
                        }
                    }
                }
            }
        }
    }

    fun setMealCount(mealCount: Int) {
        viewModelScope.launch { userPreferencesRepository.setMealCount(mealCount) }

        _uiState.update { it.setMealCount(mealCount) }
    }

    private fun addUser(user: User) {
        // Check for duplicate.
        if (_uiState.value.containsUser(user)) {
            Log.e(TAG, "User already exists")
            return
        }

        viewModelScope.launch {
            userPreferencesRepository.addUser(user)
        }

        _uiState.update { it.addUser(user) }

        viewModelScope.launch(Dispatchers.IO) {
            val uri = localUserAvatarRepository.fetchAndSaveUserImage(user.name)
            if (uri != null) {
                val updatedUser = user.copy(imageUri = uri)
                userPreferencesRepository.editUser(updatedUser)
                withContext(Dispatchers.Main) {
                    _uiState.update { it.editUser(updatedUser) }
                }
            }
        }
    }

//    fun addUser(name: String) { // Todo: Implement
//        // Check for duplicate.
//        if (_uiState.value.containsUser(name)) {
//            Log.e(TAG, "User already exists")
//            return
//        }
//
//        val user = User(name = name)
//
//        viewModelScope.launch { userPreferencesRepository.addUser(user) }
//
//        _uiState.update { it.addUser(user) }
//    }

    fun removeUser(user: User) {
        // Check if exists
        if (!_uiState.value.containsUser(user)) {
            Log.e(TAG, "User does not exist")
            return
        }

        viewModelScope.launch {
            userPreferencesRepository.removeUser(user)
            localUserAvatarRepository.deleteUserAvatar(user.name)
        }

        _uiState.update { it.removeUser(user) }
    }

    fun editUser(user: User) { // Todo: edit name or edit desired ratio
        // Check if exists
        val initialUser = _uiState.value.userList.firstOrNull { (u, _) -> u.id == user.id }?.first
        if (initialUser == null) {
            Log.e(TAG, "User does not exist")
            addUser(user)
            return
        }

        viewModelScope.launch { userPreferencesRepository.editUser(user) }

        _uiState.update { it.editUser(user).updatePercentage().updateWeight() }

        if (initialUser.name != user.name) {
            viewModelScope.launch(Dispatchers.IO) {
                localUserAvatarRepository.deleteUserAvatar(initialUser.name)
                val uri = localUserAvatarRepository.fetchAndSaveUserImage(user.name)
                if (uri != null) {
                    val updatedUser = user.copy(imageUri = uri)
                    userPreferencesRepository.editUser(updatedUser)
                    withContext(Dispatchers.Main) {
                        _uiState.update { it.editUser(updatedUser) }
                    }
                }
            }
        }
    }

    fun addPlate(plate: Plate) { // Todo: Is this really necessary?
        viewModelScope.launch {
            val updatedPlate = if (plate.imageUri != null) {
                localPlateImageRepository.savePlateImage(
                    plateUUID = plate.id.toString(),
                    imageUri = plate.imageUri
                ).let { savedImage ->
                    plate.copy(imageUri = savedImage)
                }
            } else plate
            _uiState.update { it.addPlate(updatedPlate) }
            userPreferencesRepository.savePlates(uiState.value.plateList)
        }
    }

    fun removePlate(plate: Plate) {
        _uiState.update { it.removePlate(plate) }
        viewModelScope.launch {
            localPlateImageRepository.deletePlateImage(plate.id.toString())
            userPreferencesRepository.savePlates(uiState.value.plateList)
        }
    }

    fun editPlate(plate: Plate) {
        val initialPlate = _uiState.value.plateList.firstOrNull { it.id == plate.id }
        if (initialPlate == null) {
            addPlate(plate)
            return
        }

        viewModelScope.launch {
            val updatedPlate =
                if (plate.imageUri != null && plate.imageUri != initialPlate.imageUri) {
                    val savedImage = localPlateImageRepository.savePlateImage(
                        plateUUID = plate.id.toString(),
                        imageUri = plate.imageUri
                    )
                    plate.copy(imageUri = savedImage)
                } else plate
            _uiState.update { it.editPlate(updatedPlate) }
            userPreferencesRepository.savePlates(uiState.value.plateList)
        }
    }

    fun reorderPlates(from: Int, to: Int) {
        if (from == to) return

        val plateList = _uiState.value.plateList.toMutableList()
        val plate = plateList.removeAt(from)
        plateList.add(to, plate)
        _uiState.update { it.copy(plateList = plateList) }
        viewModelScope.launch { userPreferencesRepository.savePlates(uiState.value.plateList) }
    }

    fun setActivePlate(plate: Plate) {
        if (_uiState.value.activePlate == plate) {
            Log.w(TAG, "Plate already active")
            return
        }

        viewModelScope.launch { userPreferencesRepository.setActivePlate(plate) }

        _uiState.update { it.setActivePlate(plate) }
    }

    fun setTotalWeight(totalWeight: Float) {
        viewModelScope.launch { userPreferencesRepository.setTotalWeight(totalWeight) }

        _uiState.update { it.setTotalWeight(totalWeight) }
    }

    fun changeRatioMode(ratioMode: RatioMode) {
        viewModelScope.launch { userPreferencesRepository.changeRatioMode(ratioMode) }

        _uiState.update { it.changeRatioMode(ratioMode) }

        viewModelScope.launch { // Todo: Replace complete list.
            _uiState.value.userList.forEach { (user, _) ->
                userPreferencesRepository.editUser(user)
            }
        }
    }

    private fun calculateWeight(
        totalWeight: Float,
        plateWeight: Float,
        userPercentage: Int,
        mealCount: Int
    ) = ((totalWeight - plateWeight) * userPercentage / 100 / mealCount)

    companion object {
        private val TAG: String? = PortioViewModel::class.simpleName
    }
}

data class UiState(
    val plateList: List<Plate> = emptyList(),
    val mealCount: Int = 0,
    val userList: List<Pair<User, Float>> = emptyList(),
    val activePlate: Plate? = null, // Todo: Must change to an index
    val totalWeight: Float = 0f,
    val ratioMode: RatioMode = RatioMode.EQUALLY
)

class PortioViewModelFactory(
    private val application: Application,
    private val userPreferencesRepository: UserPreferencesRepository,
    private val localPlateImageRepository: LocalPlateImageRepository,
    private val localUserAvatarRepository: LocalUserAvatarRepository
) : ViewModelProvider.Factory {
    override fun <T : ViewModel> create(modelClass: Class<T>): T {
        return PortioViewModel(
            application = application,
            userPreferencesRepository = userPreferencesRepository,
            localPlateImageRepository = localPlateImageRepository,
            localUserAvatarRepository = localUserAvatarRepository
        ) as T
    }
}

enum class RatioMode {
    EQUALLY, PARTS, AMOUNT
}