package com.portio.data.repository

import android.content.Context
import android.net.Uri
import android.util.Log
import com.portio.utils.cropResizeRotateAndSaveImage
import java.io.File

interface PlateImageRepository {
    suspend fun savePlateImage(plateUUID: String, imageUri: Uri): Uri
    suspend fun deletePlateImage(plateUUID: String)
}

class LocalPlateImageRepository(
    private val localDataSource: LocalPlateImageDataSource
) : PlateImageRepository {

    override suspend fun savePlateImage(plateUUID: String, imageUri: Uri): Uri {
        return localDataSource.saveImage(plateUUID, imageUri)
    }

    override suspend fun deletePlateImage(plateUUID: String) {
        localDataSource.deleteImages(plateUUID)
    }
}

class LocalPlateImageDataSource(private val context: Context) {

    fun saveImage(plateUUID: String, imageUri: Uri): Uri {
        val plateDir = getPlateDirectory(plateUUID)

        plateDir.listFiles()?.forEach {
            Log.i(TAG, "Deleting file ${it.name}")
            it.delete()
        }

        val timestamp = System.currentTimeMillis()
        val newFile = File(plateDir, "${timestamp}.png")

        cropResizeRotateAndSaveImage(
            context = context,
            uri = imageUri,
            newFile = newFile,
            maxDp = 200f
        ).let {
            if (it) Log.i(TAG, "Saved image to $newFile")
            else Log.e(TAG, "Failed to save image")
        }

        return Uri.fromFile(newFile)
    }

    fun deleteImages(plateUUID: String) {
        val plateDir = getPlateDirectory(plateUUID)

        plateDir.listFiles()?.forEach {
            Log.i(TAG, "Deleting file ${it.name}")
        }

        plateDir.deleteRecursively()
    }


    private fun getPlateDirectory(plateUUID: String): File {
        val dir = File(context.filesDir, "PlateImages/$plateUUID")
        if (!dir.exists()) dir.mkdirs()
        return dir
    }

    companion object {
        private val TAG: String? = LocalPlateImageDataSource::class.simpleName
    }
}