package com.portio.data.repository

import androidx.datastore.core.DataStore
import androidx.datastore.preferences.core.Preferences
import androidx.datastore.preferences.core.edit
import androidx.datastore.preferences.core.floatPreferencesKey
import androidx.datastore.preferences.core.intPreferencesKey
import androidx.datastore.preferences.core.stringPreferencesKey
import androidx.datastore.preferences.core.stringSetPreferencesKey
import com.portio.RatioMode
import com.portio.data.Plate
import com.portio.data.User
import kotlinx.coroutines.flow.first
import kotlinx.serialization.json.Json


data class UserPreferences(
    val plates: List<Plate>?,
    val users: List<User>?,
    val mealCount: Int,
    val totalWeight: Float,
    val activePlate: Plate?,
    val ratioMode: RatioMode
)

class UserPreferencesRepository(private val dataStore: DataStore<Preferences>) {

    private object PreferencesKeys {
        // PLATES_LEGACY should not be used anymore, but is kept for backwards compatibility
        // The Set type does not guarantee the order of the list.
        val PLATES_LEGACY = stringSetPreferencesKey("plates")
        val PLATES_V2 = stringPreferencesKey("plates_v2")
        val USERS = stringSetPreferencesKey("users")
        val MEAL_COUNT = intPreferencesKey("meal_count")
        val TOTAL_WEIGHT = floatPreferencesKey("total_weight")
        val ACTIVE_PLATE = stringPreferencesKey("active_plate")
        val RATIO_MODE = stringPreferencesKey("ratio_mode")
    }

    private suspend fun migrateLegacyPreferences() {
        dataStore.edit { preferences ->
            val plates = preferences[PreferencesKeys.PLATES_LEGACY]
            if (plates != null) {
                val plateList = plates.map { Json.decodeFromString<Plate>(it) }.toList()
                preferences[PreferencesKeys.PLATES_V2] = Json.encodeToString(plateList)
                preferences.remove(PreferencesKeys.PLATES_LEGACY)
            }
        }
    }

    suspend fun fetchInitialPreferences(): UserPreferences {
        migrateLegacyPreferences()
        return mapUserPreferences(dataStore.data.first().toPreferences())
    }

    private fun mapUserPreferences(preferences: Preferences): UserPreferences {
        val plates = preferences[PreferencesKeys.PLATES_V2]
            ?.let { Json.decodeFromString<List<Plate>>(it) }
        val users = preferences[PreferencesKeys.USERS]
            ?.map { Json.decodeFromString<User>(it) }
        val mealCount = preferences[PreferencesKeys.MEAL_COUNT] ?: 2
        val totalWeight = preferences[PreferencesKeys.TOTAL_WEIGHT] ?: 2500f
        val activePlate = preferences[PreferencesKeys.ACTIVE_PLATE]
            ?.let { Json.decodeFromString<Plate>(it) }
        val ratioMode = preferences[PreferencesKeys.RATIO_MODE]
            ?.let { Json.decodeFromString<RatioMode>(it) } ?: RatioMode.EQUALLY

        return UserPreferences(
            plates,
            users,
            mealCount,
            totalWeight,
            activePlate,
            ratioMode
        )
    }

    suspend fun setTotalWeight(totalWeight: Float) {
        dataStore.edit { preferences ->
            preferences[PreferencesKeys.TOTAL_WEIGHT] = totalWeight
        }
    }

    suspend fun setMealCount(mealCount: Int) {
        dataStore.edit { preferences ->
            preferences[PreferencesKeys.MEAL_COUNT] = mealCount
        }
    }

    suspend fun savePlates(plates: List<Plate>?) {
        dataStore.edit { preferences ->
            preferences[PreferencesKeys.PLATES_V2] = Json.encodeToString(plates ?: emptyList())
        }
    }

    suspend fun addUser(user: User) {
        dataStore.edit { preferences ->
            val users = preferences[PreferencesKeys.USERS] ?: emptySet()
            preferences[PreferencesKeys.USERS] = users.plus(Json.encodeToString(user))
        }
    }

    suspend fun removeUser(user: User) {
        dataStore.edit { preferences ->
            val users = preferences[PreferencesKeys.USERS]?.map { Json.decodeFromString<User>(it) }
                ?: emptyList()
            preferences[PreferencesKeys.USERS] = users.filter { it.id != user.id }
                .map { Json.encodeToString(it) }.toSet()
        }
    }

    suspend fun editUser(user: User) {
        dataStore.edit { preferences ->
            val users = preferences[PreferencesKeys.USERS]?.map { Json.decodeFromString<User>(it) }
                ?: emptyList()
            preferences[PreferencesKeys.USERS] = users.map {
                if (it.id == user.id) Json.encodeToString(user)
                else Json.encodeToString(it)
            }.toSet()
        }
    }

    suspend fun setActivePlate(plate: Plate) {
        dataStore.edit { preferences ->
            preferences[PreferencesKeys.ACTIVE_PLATE] = Json.encodeToString(plate)
        }
    }

    suspend fun changeRatioMode(ratioMode: RatioMode) {
        dataStore.edit { preferences ->
            preferences[PreferencesKeys.RATIO_MODE] = Json.encodeToString(ratioMode)
        }
    }

    companion object {
        private val TAG = UserPreferencesRepository::class.simpleName
    }
}