package com.portio.ui.screens.home

import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.WindowInsets
import androidx.compose.foundation.layout.asPaddingValues
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.systemBars
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Done
import androidx.compose.material.icons.filled.MoreVert
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.Button
import androidx.compose.material3.DropdownMenu
import androidx.compose.material3.DropdownMenuItem
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.material3.TextButton
import androidx.compose.material3.TextField
import androidx.compose.material3.TopAppBar
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.TextRange
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.input.TextFieldValue
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import com.portio.R
import com.portio.UiState
import com.portio.data.Plate
import com.portio.utils.UnitLocale

@Composable
fun HomeScreen(
    uiState: UiState,
    onWeightChanged: (Float) -> Unit,
    onMealCountChanged: (Int) -> Unit,
    onActivePlateChanged: (Plate) -> Unit,
    onManageUserClick: () -> Unit,
    onManagePlateClick: () -> Unit,
    modifier: Modifier = Modifier
) {
    // Dialog to display when modifying the total weight.
    val showChangeWeightDialog = remember { mutableStateOf(false) }
    if (showChangeWeightDialog.value) ChangeWeightDialog(
        initialValue = uiState.totalWeight,
        onDismiss = { showChangeWeightDialog.value = false },
        onWeightChanged = {
            onWeightChanged(it)
            showChangeWeightDialog.value = false
        }
    )

    // Dialog to display when modifying the meal count.
    val showChangeMealCountDialog = remember { mutableStateOf(false) }
    if (showChangeMealCountDialog.value) ChangeMealCountDialog(
        initialValue = uiState.mealCount,
        onDismiss = { showChangeMealCountDialog.value = false },
        onMealCountChanged = {
            onMealCountChanged(it)
            showChangeMealCountDialog.value = false
        }
    )

    Scaffold(
        modifier = modifier,
        contentWindowInsets = WindowInsets(bottom = 0),
        topBar = {
            PortioTopAppBar(
                onManageUserClick = onManageUserClick,
                onManagePlateClick = onManagePlateClick
            )
        }
    ) { innerPadding ->
        Column(
            modifier = Modifier.padding(innerPadding),
        ) {
            WeightIndicator(
                modifier = Modifier.padding(horizontal = 8.dp),
                totalWeight = uiState.totalWeight,
                onClick = { showChangeWeightDialog.value = true }
            )
            PlateCarousel(
                modifier = Modifier.padding(vertical = 32.dp),
                initialPlate = uiState.plateList.indexOf(uiState.activePlate),
                plateList = uiState.plateList,
                onSelectedPlate = onActivePlateChanged,
                onManagePlateClick = onManagePlateClick
            )

            Surface(
                shape = RoundedCornerShape(32.dp, 32.dp, 0.dp, 0.dp),
                tonalElevation = 1.dp,
                shadowElevation = 1.dp
            ) {
                Column(
                    Modifier
                        .weight(1f)
                        .fillMaxHeight()
                        .padding(vertical = 16.dp)
                        .padding(
                            bottom = WindowInsets.systemBars
                                .asPaddingValues()
                                .calculateBottomPadding()
                        )
                ) {
                    PortionHeader(
                        modifier = Modifier.padding(bottom = 16.dp),
                        mealCount = uiState.mealCount,
                        personCount = uiState.userList.count(),
                        portionCount = uiState.userList.count() * uiState.mealCount,
                        onMealCountClicked = { showChangeMealCountDialog.value = true }
                    )
                    LazyColumn {
                        items(uiState.userList.count()) { index ->
                            UserCard(
                                user = uiState.userList[index].first,
                                weight = uiState.userList[index].second,
                                modifier = Modifier.padding(horizontal = 12.dp, vertical = 4.dp),
                                onClick = onManageUserClick
                            )
                        }
                        if (uiState.userList.isEmpty())
                            item {
                                Text(
                                    text = stringResource(R.string.no_user),
                                    modifier = Modifier
                                        .fillMaxWidth()
                                        .padding(horizontal = 8.dp, vertical = 4.dp),
                                    textAlign = TextAlign.Center,
                                )
                            }
                        item {
                            TextButton(
                                onClick = onManageUserClick,
                                modifier = Modifier
                                    .fillMaxWidth()
                                    .padding(horizontal = 8.dp),
                            ) {
                                Icon(
                                    painter = painterResource(id = android.R.drawable.ic_input_add),
                                    contentDescription = null,
                                )
                                Text(
                                    text = stringResource(R.string.manage_users),
                                    modifier = Modifier.align(Alignment.CenterVertically)
                                )
                            }
                        }
                    }


                }
            }
        }
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun PortioTopAppBar(
    onManageUserClick: () -> Unit,
    onManagePlateClick: () -> Unit
) {
    TopAppBar(title = {}, actions = {

        val showDropDownMenu = remember { mutableStateOf(false) }

        IconButton(
            onClick = { showDropDownMenu.value = true },
        ) {
            Icon(
                imageVector = Icons.Filled.MoreVert,
                contentDescription = stringResource(R.string.menu),
                tint = MaterialTheme.colorScheme.primary
            )
        }

        if (showDropDownMenu.value) {
            DropdownMenu(
                expanded = true,
                onDismissRequest = { showDropDownMenu.value = false }) {
                DropdownMenuItem( // Manage Users
                    leadingIcon = {
                        Icon(
                            painter = painterResource(id = R.drawable.baseline_group_add_24),
                            contentDescription = null,
                            tint = MaterialTheme.colorScheme.primary
                        )
                    },
                    text = {
                        Text(
                            text = stringResource(R.string.manage_users),
                            color = MaterialTheme.colorScheme.primary
                        )
                    },
                    onClick = {
                        showDropDownMenu.value = false
                        onManageUserClick()
                    }
                )
                DropdownMenuItem( // Manage Plates
                    leadingIcon = {
                        Icon(
                            painter = painterResource(id = R.drawable.baseline_library_add_24),
                            contentDescription = null,
                            tint = MaterialTheme.colorScheme.primary
                        )
                    },
                    text = {
                        Text(
                            text = stringResource(R.string.manage_plates),
                            color = MaterialTheme.colorScheme.primary
                        )
                    },
                    onClick = {
                        showDropDownMenu.value = false
                        onManagePlateClick()
                    }
                )
            }
        }
    })
}

@Composable
fun ChangeMealCountDialog(
    initialValue: Int,
    onDismiss: () -> Unit,
    onMealCountChanged: (Int) -> Unit,
    modifier: Modifier = Modifier,
) {
    var mealCount by remember { mutableIntStateOf(initialValue) }
    val isError = mealCount <= 0

    var textFieldSelection by remember {
        mutableStateOf(TextRange(mealCount.toString().length))
    }

    AlertDialog(
        modifier = modifier,
        onDismissRequest = onDismiss,
        title = { Text(text = stringResource(R.string.change_meal_count)) },
        text = {
            Column {
                Text(
                    text = stringResource(R.string.change_meal_count_description),
                    modifier = Modifier
                        .fillMaxWidth()
                        .padding(bottom = 8.dp)
                )
                TextField(
                    value = run {
                        val text = mealCount.toString()
                        if (text == "0") { // Force cursor to be at the end
                            textFieldSelection = TextRange(1)
                        }
                        TextFieldValue(text = text, selection = textFieldSelection)
                    },
                    isError = mealCount < 0,
                    keyboardOptions = KeyboardOptions(
                        keyboardType = KeyboardType.Number,
                        imeAction = ImeAction.Done
                    ),
                    onValueChange = { textFieldValue ->
                        val textFieldText = textFieldValue.text.filter { txt -> txt.isDigit() }
                        textFieldSelection = textFieldValue.selection
                        mealCount = if (textFieldText.isNotEmpty()) textFieldText.toInt() else 0
                    },
                    label = { Text(text = stringResource(R.string.meal_count)) },
                    modifier = Modifier.fillMaxWidth()
                )
            }
        },
        dismissButton = {
            TextButton(onClick = onDismiss) {
                Text(stringResource(R.string.cancel))
            }
        },
        confirmButton = {
            Button(
                enabled = !isError,
                onClick = {
                    onMealCountChanged(mealCount)
                }
            ) {
                Icon(
                    imageVector = Icons.Filled.Done,
                    contentDescription = "null",
                )
                Text(
                    text = stringResource(R.string.done),
                    modifier = Modifier.padding(start = 8.dp)
                )
            }
        }
    )
}

@Composable
fun ChangeWeightDialog(
    initialValue: Float,
    onDismiss: () -> Unit,
    onWeightChanged: (Float) -> Unit,
    modifier: Modifier = Modifier
) {
    var weight by remember { mutableIntStateOf(initialValue.toInt()) }
    val isError = weight <= 0

    var textFieldSelection by remember {
        mutableStateOf(TextRange(weight.toString().length))
    }
    AlertDialog(
        modifier = modifier,
        onDismissRequest = onDismiss,
        title = { Text(text = stringResource(R.string.change_total_weight)) },
        text = {
            val stringUnit =
                if (UnitLocale.default == UnitLocale.Metric)
                    stringResource(R.string.gram_unit)
                else stringResource(R.string.ounce_unit)
            Column {
                Text(
                    text = stringResource(R.string.total_weight_description),
                    modifier = Modifier
                        .fillMaxWidth()
                        .padding(bottom = 8.dp)
                )
                TextField(
                    value = run {
                        val text = weight.toString()
                        if (text == "0") { // Force cursor to be at the end
                            textFieldSelection = TextRange(1)
                        }
                        TextFieldValue(text = text, selection = textFieldSelection)
                    },
                    isError = weight < 0,
                    keyboardOptions = KeyboardOptions(
                        keyboardType = KeyboardType.Number,
                        imeAction = ImeAction.Done
                    ),
                    onValueChange = { textFieldValue ->
                        val textFieldText = textFieldValue.text.filter { txt -> txt.isDigit() }
                        textFieldSelection = textFieldValue.selection
                        weight = if (textFieldText.isNotEmpty()) textFieldText.toInt() else 0
                    },
                    label = {
                        Text(text = stringResource(R.string.total_weight_with_unit, stringUnit))
                    },
                    modifier = Modifier.fillMaxWidth()
                )
            }
        },
        dismissButton = {
            TextButton(onClick = onDismiss) {
                Text(stringResource(R.string.cancel))
            }
        },
        confirmButton = {
            Button(
                enabled = !isError,
                onClick = {
                    onWeightChanged(weight.toFloat())
                },
            ) {
                Icon(
                    imageVector = Icons.Filled.Done,
                    contentDescription = "null",
                )
                Text(
                    text = stringResource(R.string.done),
                    modifier = Modifier.padding(start = 8.dp)
                )
            }
        }
    )
}