package com.portio.ui.screens.home

import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.layout.wrapContentHeight
import androidx.compose.foundation.pager.PagerDefaults
import androidx.compose.foundation.pager.PagerSnapDistance
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.graphicsLayer
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.util.lerp
import androidx.compose.foundation.pager.HorizontalPager
import androidx.compose.foundation.pager.PagerState
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.remember
import androidx.compose.ui.platform.LocalConfiguration
import com.portio.data.Plate
import com.portio.ui.sharedcomponents.PlateCardButton
import kotlin.math.absoluteValue
import kotlin.math.min

@Preview
@Composable
fun PlateCarouselPreview() {
    val plateList = listOf(
        Plate("Plate 1", 1.25f),
        Plate("Plate 2", 2.5f),
        Plate("Plate 3", 5.0f),
        Plate("Plate 4", 10.0f),
        Plate("Plate 5", 20.0f),
    )
    PlateCarousel(Modifier, plateList, { _ -> })
}

@Composable
fun PlateCarousel(
    modifier: Modifier,
    plateList: List<Plate>,
    onSelectedPlate: (Plate) -> Unit,
    initialPlate: Int = 0,
    onManagePlateClick: () -> Unit = {}
) {
    val pagerKey = plateList.hashCode() // Update pagerState if plateList changes
    val pagerState = remember(pagerKey) {
        if (plateList.isEmpty()) PagerState(currentPage = 0, pageCount = { 1 })
        else PagerState(
            currentPage = run {
                // Try to center the initial plate in the carousel
                val pageCount = 500
                val rightShift = (initialPlate - pageCount / 2) % plateList.size
                val leftShift = plateList.size - rightShift
                val bestChoice = // Define the best choice to stay the closest to the center (250/2)
                    if (min(rightShift, leftShift) == rightShift) rightShift else -leftShift
                val centeredPage = pageCount / 2 + bestChoice
                centeredPage
            },
            pageCount = { 500 }
        )
    }

    LaunchedEffect(pagerState.currentPage) {
        if (plateList.isNotEmpty())
            onSelectedPlate(plateList[pagerState.currentPage % plateList.size])
    }
    val fling = PagerDefaults.flingBehavior(
        state = pagerState,
        pagerSnapDistance = PagerSnapDistance.atMost(2)
    )

    val pageWidth = 190.dp
    val contentPadding = (LocalConfiguration.current.screenWidthDp.dp - pageWidth) / 2
    HorizontalPager(
        modifier = modifier
            .fillMaxWidth()
            .wrapContentHeight(),
        state = pagerState,
        flingBehavior = fling,
        contentPadding = PaddingValues(horizontal = contentPadding),
    ) { page ->
        val cardModifier = Modifier.graphicsLayer {
            // Decomposing the calculation into intermediate steps
            val currentPage = pagerState.currentPage
            val currentPageOffsetFraction = pagerState.currentPageOffsetFraction
            val pageOffset = (currentPage - page) + currentPageOffsetFraction
            val absolutePageOffset = pageOffset.absoluteValue

            // Scale and alpha calculations
            lerp(
                start = 0.85f,
                stop = 1f,
                fraction = 1f - absolutePageOffset.coerceIn(0f, 1f)
            ).also { scale ->
                scaleX = scale
                scaleY = scale
            }
            alpha = lerp(
                start = 0.5f,
                stop = 1f,
                fraction = 1f - absolutePageOffset.coerceIn(0f, 1f)
            )
        }
        PlateCardButton(
            modifier = cardModifier.width(pageWidth),
            plate = if (plateList.isEmpty()) null else plateList[page % plateList.size],
            onClick = onManagePlateClick
        )
    }
}