package com.portio.ui.screens.home

import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.IntrinsicSize
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.layout.wrapContentWidth
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.material3.TextButton
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.layout.SubcomposeLayout
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.Constraints
import androidx.compose.ui.unit.dp
import com.portio.R

@Composable
@Preview(showBackground = true)
fun PortionHeaderPreview() {
    PortionHeader(mealCount = 3, personCount = 2, portionCount = 6)
}

@Composable
fun Meal(mealCount: Int, onMealCountClicked: () -> Unit = {}) {
    TextButton(modifier = Modifier.wrapContentWidth(), onClick = onMealCountClicked) {
        Column(horizontalAlignment = Alignment.CenterHorizontally) {
            Text(
                text = stringResource(R.string.meal),
                style = MaterialTheme.typography.bodyLarge,
                textAlign = TextAlign.Center
            )
            Text(
                modifier = Modifier.padding(top = 4.dp),
                text = mealCount.toString(),
                style = MaterialTheme.typography.titleLarge.copy(fontWeight = FontWeight.Bold),
                textAlign = TextAlign.Center
            )
        }
    }
}

@Composable
fun Operator(operator: String) {
    Column(
        verticalArrangement = Arrangement.Bottom,
        modifier = Modifier
            .fillMaxHeight()
            .width(10.dp)
            .padding(bottom = ButtonDefaults.TextButtonContentPadding.calculateBottomPadding())
    ) {
        Text(
            modifier = Modifier.fillMaxWidth(),
            text = operator,
            textAlign = TextAlign.Center,
            style = MaterialTheme.typography.titleLarge
        )
    }
}

@Composable
fun Person(personCount: Int) {
    Column(
        modifier = Modifier
            .padding(ButtonDefaults.TextButtonContentPadding)
            .width(IntrinsicSize.Min)
    ) {
        Text(
            modifier = Modifier.fillMaxWidth(),
            text = stringResource(R.string.people),
            style = MaterialTheme.typography.bodyLarge,
            textAlign = TextAlign.Center
        )
        Text(
            modifier = Modifier
                .fillMaxWidth()
                .padding(top = 4.dp),
            text = personCount.toString(),
            style = MaterialTheme.typography.titleLarge.copy(fontWeight = FontWeight.Bold),
            textAlign = TextAlign.Center
        )
    }
}

@Composable
fun Portion(portionCount: Int) {
    Column(
        modifier = Modifier.padding(ButtonDefaults.TextButtonContentPadding)
    ) {
        Text(
            modifier = Modifier.fillMaxWidth(),
            text = stringResource(R.string.portions),
            style = MaterialTheme.typography.bodyLarge,
            textAlign = TextAlign.Center
        )
        Text(
            modifier = Modifier
                .fillMaxWidth()
                .padding(top = 4.dp),
            text = portionCount.toString(),
            style = MaterialTheme.typography.titleLarge.copy(fontWeight = FontWeight.Bold),
            textAlign = TextAlign.Center
        )
    }
}

/**
 * Note: This layout is more complex than necessary for this use case.
 * It is intended to demonstrate the use of SubcomposeLayout.
 */
@Composable
fun PortionHeader(
    mealCount: Int,
    personCount: Int,
    portionCount: Int,
    modifier: Modifier = Modifier,
    onMealCountClicked: () -> Unit = {},
) {

    SubcomposeLayout(
        modifier = modifier.fillMaxWidth()
    ) { _ -> // Todo: Use constraints

        val mealMeasurable1 = subcompose(1) { Meal(mealCount, onMealCountClicked) }
        val mealMeasurable2 = subcompose(11) { Meal(mealCount, onMealCountClicked) }
        val multiplyOperatorMeasurable = subcompose(2) { Operator("×") }
        val personMeasurable = subcompose(3) { Person(personCount) }
        val equalOperatorMeasurable = subcompose(4) { Operator("=") }
        val portionMeasurable1 = subcompose(5) { Portion(portionCount) }
        val portionMeasurable2 = subcompose(55) { Portion(portionCount) }

        // Need to know the max width between c1 and c5
        val mealMeasure = mealMeasurable1.map { it.measure(Constraints()) }
        val portionMeasure = portionMeasurable1.map { it.measure(Constraints()) }
        val maxWidth = maxOf(mealMeasure.maxOf { it.width }, portionMeasure.maxOf { it.width })
        val maxHeight = maxOf(mealMeasure.maxOf { it.height }, portionMeasure.maxOf { it.height })

        val mealPlaceable =
            mealMeasurable2.map {
                it.measure(
                    Constraints(
                        minWidth = maxWidth,
                        maxWidth = maxWidth
                    )
                )
            }
        val multiplyOperatorPlaceable =
            multiplyOperatorMeasurable.map {
                it.measure(
                    Constraints(
                        minHeight = maxHeight,
                        maxHeight = maxHeight
                    )
                )
            }
        val personPlaceable = personMeasurable.map { it.measure(Constraints()) }
        val equalOperatorPlaceable =
            equalOperatorMeasurable.map {
                it.measure(
                    Constraints(
                        minHeight = maxHeight,
                        maxHeight = maxHeight
                    )
                )
            }
        val portionPlaceable =
            portionMeasurable2.map {
                it.measure(
                    Constraints(
                        minWidth = maxWidth,
                        maxWidth = maxWidth
                    )
                )
            }

        val totalWidth = mealPlaceable.maxOf { it.width } +
                multiplyOperatorPlaceable.maxOf { it.width } +
                personPlaceable.maxOf { it.width } +
                equalOperatorPlaceable.maxOf { it.width } +
                portionPlaceable.maxOf { it.width }

        val totalHeight = maxOf(mealPlaceable.maxOf { it.height },
            multiplyOperatorPlaceable.maxOf { it.height },
            personPlaceable.maxOf { it.height },
            equalOperatorPlaceable.maxOf { it.height },
            portionPlaceable.maxOf { it.height })

        layout(totalWidth, totalHeight) {
            mealPlaceable.forEach {
                it.place(0, 0)
            }
            val mealX = mealPlaceable.maxOf { it.width }
            multiplyOperatorPlaceable.forEach {
                it.place(mealX, 0)
            }
            val multiplyOperatorX = multiplyOperatorPlaceable.maxOf { it.width } + mealX
            personPlaceable.forEach {
                it.place(multiplyOperatorX, 0)
            }
            val personX = personPlaceable.maxOf { it.width } + multiplyOperatorX
            equalOperatorPlaceable.forEach {
                it.place(personX, 0)
            }
            val equalOperatorX = equalOperatorPlaceable.maxOf { it.width } + personX
            portionPlaceable.forEach {
                it.place(equalOperatorX, 0)
            }
        }
    }
}