package com.portio.ui.screens.home

import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.IntrinsicSize
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.material3.Card
import androidx.compose.material3.CardDefaults
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.material3.surfaceColorAtElevation
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.alpha
import androidx.compose.ui.draw.clip
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import coil.compose.AsyncImage
import com.portio.R
import com.portio.data.User
import com.portio.utils.UnitLocale
import java.net.URLDecoder
import java.nio.charset.StandardCharsets

@Composable
@Preview(showBackground = true)
fun UserCardPreview() {
    val user = User(
        name = "Anaïs",
        effectiveRatio = 65,
    )
    UserCard(
        user = user,
        weight = 350.50f,
        modifier = Modifier
            .padding(8.dp)
            .width(300.dp)
    )
}

@Composable
fun UserCard(
    modifier: Modifier = Modifier,
    user: User,
    weight: Float,
    onClick: () -> Unit = {}
) {

    Card(
        modifier = modifier.height(IntrinsicSize.Max),
        elevation = CardDefaults.elevatedCardElevation(),
        colors = CardDefaults.elevatedCardColors(
            containerColor = MaterialTheme.colorScheme.surfaceColorAtElevation(2.dp)
        ),
        onClick = onClick
    ) {
        Row(
            modifier = Modifier.padding(8.dp)
        ) {

            if (user.imageUri == null) {
                Icon(
                    painter = painterResource(id = R.drawable.baseline_image_not_supported_24),
                    contentDescription = null,
                    modifier = Modifier
                        .align(Alignment.CenterVertically)
                        .padding(start = 8.dp)
                        .padding(end = 16.dp)
                        .size(40.dp)
                        .alpha(0.6f)
                )
            } else {
                val decodedUri = URLDecoder
                    .decode(user.imageUri.toString(), StandardCharsets.UTF_8.toString())
                AsyncImage(
                    model = decodedUri,
                    contentDescription = null,
                    modifier = Modifier
                        .align(Alignment.CenterVertically)
                        .padding(start = 8.dp)
                        .padding(end = 16.dp)
                        .size(40.dp)
                        .clip(MaterialTheme.shapes.medium),
                    contentScale = ContentScale.Crop
                )
            }

            Text(
                modifier = Modifier
                    .weight(1f)
                    .align(Alignment.CenterVertically),
                text = user.name,
                style = MaterialTheme.typography.bodyLarge
            )

            Column(
                modifier = Modifier
                    .fillMaxHeight()
                    .padding(end = 8.dp),
                verticalArrangement = Arrangement.Center
            ) {
                Text(
                    // Cast to Int to avoid floating value. May be used in the future.
                    text = stringResource(R.string.percentage, user.effectiveRatio),
                    style = MaterialTheme.typography.bodyMedium,
                    modifier = Modifier.align(Alignment.End)
                )

                Spacer(modifier = Modifier.height(6.dp))

                Text(
                    // Cast to Int to avoid floating value. May be used in the future.
                    text = if (UnitLocale.default == UnitLocale.Metric)
                        stringResource(R.string.gram, weight.toInt())
                    else stringResource(R.string.ounce, weight.toInt()),
                    style = MaterialTheme.typography.bodyMedium,
                    fontWeight = FontWeight.Bold,
                    modifier = Modifier.align(Alignment.End)
                )
            }
        }
    }
}