package com.portio.ui.screens.platemanager

import android.Manifest
import android.content.pm.PackageManager
import android.net.Uri
import android.util.Log
import androidx.activity.compose.BackHandler
import androidx.activity.compose.rememberLauncherForActivityResult
import androidx.activity.result.contract.ActivityResultContracts
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.filled.Done
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.Button
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TextButton
import androidx.compose.material3.TextField
import androidx.compose.material3.TextFieldDefaults
import androidx.compose.material3.TopAppBar
import androidx.compose.material3.surfaceColorAtElevation
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.TextRange
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.input.TextFieldValue
import androidx.compose.ui.unit.dp
import androidx.core.content.ContextCompat
import com.portio.R
import com.portio.data.Plate
import com.portio.utils.createImageFile
import com.portio.ui.sharedcomponents.PlateCard
import com.portio.utils.UnitLocale

@Composable
fun EditPlateScreen(
    plate: Plate?,
    modifier: Modifier = Modifier,
    onNavigationClicked: () -> Unit = {},
    onPlateEdited: (Plate) -> Unit = {},
) {
    val isEdit = remember { plate != null }

    // Dialog to ask if user is really sure to exit.
    val showExitDialog = remember { mutableStateOf(false) }
    if (showExitDialog.value) {
        ExitDialog(onDismiss = { exit: Boolean ->
            showExitDialog.value = false
            if (exit) onNavigationClicked()
        })
    }

    val context = LocalContext.current
    var editedPlate by remember { mutableStateOf(plate ?: Plate.default()) }
    val galleryLauncher = rememberLauncherForActivityResult(
        contract = ActivityResultContracts.GetContent(),
    ) { uri: Uri? -> uri?.let { editedPlate = editedPlate.copy(imageUri = it) } }

    val isPlateModified = !(plate?.isSimilarTo(editedPlate) ?: editedPlate.isDefault())
    BackHandler {
        if (isPlateModified) showExitDialog.value = true
        else onNavigationClicked()
    }

    var tempUri by remember { mutableStateOf<Uri?>(null) }
    val cameraLauncher = rememberLauncherForActivityResult(
        contract = ActivityResultContracts.TakePicture()
    ) { success -> if (success) editedPlate = editedPlate.copy(imageUri = tempUri) }

    val permissionLauncher = rememberLauncherForActivityResult(
        contract = ActivityResultContracts.RequestPermission(),
        onResult = { isGranted ->
            if (isGranted) {
                tempUri?.let { context.contentResolver.delete(it, null, null) }
                tempUri = createImageFile(context, editedPlate.id)
                    .apply { cameraLauncher.launch(this) }
            } else {
                Log.e("EditPlateScreen", "Permission denied")
            }
        }
    )

    Scaffold(
        modifier = modifier,
        topBar = {
            EditOrAddPlateScreenTopBar(
                isEdit = isEdit,
                onNavigationClicked = {
                    // Check if modification was done by user. If so, ask confirmation before exit.
                    if (isPlateModified) showExitDialog.value = true
                    else onNavigationClicked()
                })
        },
        floatingActionButton = {
            Button(
                onClick = {
                    onPlateEdited(editedPlate)
                    tempUri?.let { context.contentResolver.delete(it, null, null) }
                },
                enabled = editedPlate.isValid()
            ) {
                Icon(
                    imageVector = Icons.Filled.Done,
                    contentDescription = null,
                )
                Text(
                    modifier = Modifier.padding(start = 4.dp),
                    text = stringResource(R.string.done)
                )
            }
        }
    ) { innerPadding ->
        Column(
            horizontalAlignment = Alignment.CenterHorizontally,
            modifier = Modifier
                .fillMaxSize()
                .padding(innerPadding)
                .padding(top = 48.dp)
        ) {
            PlateCard(
                modifier = Modifier.padding(bottom = 32.dp),
                plate = editedPlate
            )

            val isEmptyError = editedPlate.name.isEmpty()
            val isTooLongError = editedPlate.name.length > Plate.MAX_LENGTH
            TextField(
                value = editedPlate.name,
                isError = isEmptyError || isTooLongError,
                keyboardOptions = KeyboardOptions(
                    keyboardType = KeyboardType.Text,
                    imeAction = ImeAction.Done
                ),
                supportingText = {
                    Row {
                        Text(
                            text = if (isTooLongError) stringResource(R.string.plate_name_too_long)
                            else if (isEmptyError) stringResource(R.string.plate_name_empty)
                            else "",
                        )
                        Spacer(Modifier.weight(1f))
                        Text(
                            text = stringResource(
                                R.string.size_limit,
                                editedPlate.name.length,
                                Plate.MAX_LENGTH
                            )
                        )
                    }
                },
                colors = TextFieldDefaults.colors().copy(
                    focusedContainerColor = MaterialTheme.colorScheme
                        .surfaceColorAtElevation(1.dp),
                    unfocusedContainerColor = MaterialTheme.colorScheme
                        .surfaceColorAtElevation(1.dp),
                ),
                onValueChange = { editedPlate = editedPlate.copy(name = it) },
                label = { Text(text = stringResource(R.string.plate_name)) },
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(horizontal = 12.dp)
                    .padding(bottom = 16.dp)
            )
            var textFieldSelection by remember {
                mutableStateOf(TextRange(editedPlate.weight.toInt().toString().length))
            }
            TextField(
                value = run {
                    val text = editedPlate.weight.toInt().toString()
                    if (text == "0") { // Force cursor to be at the end
                        textFieldSelection = TextRange(1)
                    }
                    TextFieldValue(text = text, selection = textFieldSelection)
                },
                isError = editedPlate.weight < 0,
                keyboardOptions = KeyboardOptions(
                    keyboardType = KeyboardType.Number,
                    imeAction = ImeAction.Done
                ),
                colors = TextFieldDefaults.colors().copy(
                    focusedContainerColor = MaterialTheme.colorScheme
                        .surfaceColorAtElevation(1.dp),
                    unfocusedContainerColor = MaterialTheme.colorScheme
                        .surfaceColorAtElevation(1.dp),
                ),
                onValueChange = { textFieldValue ->
                    val textFieldText = textFieldValue.text.filter { txt -> txt.isDigit() }
                    textFieldSelection = textFieldValue.selection
                    editedPlate = editedPlate.copy(
                        weight = if (textFieldText.isNotEmpty()) textFieldText.toFloat() else 0f
                    )
                },
                label = {
                    val stringUnit =
                        if (UnitLocale.default == UnitLocale.Metric)
                            stringResource(R.string.gram_unit)
                        else stringResource(R.string.ounce_unit)
                    Text(text = stringResource(R.string.plate_weight_with_unit, stringUnit))
                },
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(horizontal = 12.dp)
                    .padding(bottom = 16.dp)
            )
            Row(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(horizontal = 12.dp), verticalAlignment = Alignment.CenterVertically
            ) {
                Text(modifier = Modifier.weight(1f), text = stringResource(R.string.plate_image))
                IconButton(onClick = { galleryLauncher.launch("image/*") }) {
                    Icon(
                        painterResource(id = R.drawable.baseline_collections_24),
                        contentDescription = stringResource(R.string.image_from_gallery),
                        tint = MaterialTheme.colorScheme.primary
                    )
                }
                IconButton(onClick = {
                    if (ContextCompat.checkSelfPermission(
                            context,
                            Manifest.permission.CAMERA
                        ) == PackageManager.PERMISSION_GRANTED
                    ) {
                        tempUri?.let { context.contentResolver.delete(it, null, null) }
                        tempUri = createImageFile(context, editedPlate.id).apply {
                            cameraLauncher.launch(this)
                        }
                    } else {
                        permissionLauncher.launch(Manifest.permission.CAMERA)
                    }
                }) {
                    Icon(
                        painter = painterResource(id = R.drawable.baseline_camera_alt_24),
                        contentDescription = stringResource(R.string.image_from_camera),
                        tint = MaterialTheme.colorScheme.primary
                    )
                }
            }
        }
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun EditOrAddPlateScreenTopBar(
    isEdit: Boolean,
    onNavigationClicked: () -> Unit = {}
) { // Todo: Factor out
    TopAppBar(
        title = {
            Text(
                text = if (isEdit) stringResource(R.string.edit_plate)
                else stringResource(R.string.add_plate_title)
            )
        },
        navigationIcon = {
            IconButton(
                onClick = onNavigationClicked,
            ) {
                Icon(
                    imageVector = Icons.AutoMirrored.Filled.ArrowBack,
                    contentDescription = stringResource(R.string.go_back),
                    tint = MaterialTheme.colorScheme.primary
                )
            }
        })
}

@Composable
fun ExitDialog(onDismiss: (Boolean) -> Unit) {
    AlertDialog(
        onDismissRequest = { onDismiss(false) },
        title = { Text(text = stringResource(R.string.are_you_sure)) },
        text = { Text(text = stringResource(R.string.modifications_won_t_be_saved)) },
        dismissButton = {
            TextButton(onClick = { onDismiss(false) }) {
                Text(stringResource(R.string.cancel))
            }
        },
        confirmButton = {
            TextButton(onClick = { onDismiss(true) }) {
                Text(stringResource(R.string.exit))
            }
        }
    )
}