package com.portio.ui.screens.platemanager

import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.filled.Delete
import androidx.compose.material.icons.filled.Edit
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.Button
import androidx.compose.material3.Card
import androidx.compose.material3.CardDefaults
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TextButton
import androidx.compose.material3.TopAppBar
import androidx.compose.material3.surfaceColorAtElevation
import androidx.compose.runtime.Composable
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.alpha
import androidx.compose.ui.draw.clip
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.dp
import coil.compose.AsyncImage
import com.portio.R
import com.portio.data.Plate
import com.portio.utils.UnitLocale
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.material.icons.filled.Info
import androidx.compose.runtime.getValue
import androidx.compose.runtime.setValue
import androidx.compose.ui.platform.LocalView
import androidx.core.view.HapticFeedbackConstantsCompat
import androidx.core.view.ViewCompat
import sh.calvin.reorderable.ReorderableItem
import sh.calvin.reorderable.rememberReorderableLazyListState

@Composable
fun PlateManagerScreen(
    plateList: List<Plate>,
    modifier: Modifier = Modifier,
    onNavigationClicked: () -> Unit = {},
    onPlateRemoved: (Plate) -> Unit = {},
    onEditPlateClick: (Plate) -> Unit = {},
    onAddPlateClick: () -> Unit = {},
    onMove: (Int, Int) -> Unit = { _, _ -> },
) {
    var deleteDialog by remember { mutableStateOf<Plate?>(null) }
    var showInfoDialog by remember { mutableStateOf(false) }

    if (deleteDialog != null) {
        AlertDialog(
            onDismissRequest = { deleteDialog = null },
            title = { Text(text = stringResource(R.string.are_you_sure)) },
            text = { Text(text = stringResource(R.string.delete_plate_section)) },
            dismissButton = {
                TextButton(onClick = { deleteDialog = null }) {
                    Text(text = stringResource(R.string.cancel))
                }
            },
            confirmButton = {
                TextButton(
                    onClick = {
                        deleteDialog?.let {
                            onPlateRemoved(it)
                            deleteDialog = null
                        }
                    }) {
                    Text(text = stringResource(R.string.delete))
                }
            }
        )
    }

    if (showInfoDialog) {
        AlertDialog(
            onDismissRequest = { showInfoDialog = false },
            title = { Text(text = stringResource(R.string.manage_plates_info_title)) },
            text = { Text(text = stringResource(R.string.manage_plates_description)) },
            confirmButton = {
                Button(onClick = { showInfoDialog = false }) {
                    Text(text = stringResource(android.R.string.ok))
                }
            }
        )
    }

    Scaffold(modifier = modifier,
        topBar = {
            PlateManagerScreenTopBar(
                onNavigationClicked = onNavigationClicked,
                onInfoClicked = { showInfoDialog = true }
            )
        },
        floatingActionButton = {
            Button(
                onClick = onAddPlateClick,
                modifier = Modifier
            ) {
                Icon(
                    painter = painterResource(id = android.R.drawable.ic_input_add),
                    contentDescription = null
                )
                Text(
                    modifier = Modifier.padding(start = 4.dp),
                    text = stringResource(R.string.add_plate)
                )
            }
        }
    ) { innerPadding ->
        if (plateList.isEmpty())
            Text(
                text = stringResource(R.string.no_plate),
                modifier = Modifier
                    .padding(innerPadding)
                    .fillMaxWidth()
                    .padding(horizontal = 12.dp),
                style = MaterialTheme.typography.bodyLarge
            )
        else {
            val lazyListState = rememberLazyListState()
            val view = LocalView.current
            val state = rememberReorderableLazyListState(lazyListState) { from, to ->
                onMove(from.index, to.index)
                ViewCompat.performHapticFeedback(
                    view,
                    HapticFeedbackConstantsCompat.SEGMENT_FREQUENT_TICK
                )
            }
            LazyColumn(
                state = lazyListState,
                modifier = Modifier
                    .fillMaxSize()
                    .padding(innerPadding)
                    .padding(bottom = 32.dp)
                    .padding(horizontal = 12.dp)
            ) {
                items(plateList, { item -> item.id }) { index ->
                    ReorderableItem(state, index.id) { _ ->
                        PlateCard(modifier = Modifier
                            .longPressDraggableHandle(
                                onDragStarted = {
                                    ViewCompat.performHapticFeedback(
                                        view,
                                        HapticFeedbackConstantsCompat.GESTURE_START
                                    )
                                },
                                onDragStopped = {
                                    ViewCompat.performHapticFeedback(
                                        view,
                                        HapticFeedbackConstantsCompat.GESTURE_END
                                    )
                                }
                            )
                            .fillMaxWidth()
                            .padding(vertical = 4.dp),
                            plate = index,
                            onDeleteClick = { deleteDialog = index },
                            onEditClick = { onEditPlateClick(index) }
                        )
                    }
                }
            }
        }
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun PlateManagerScreenTopBar(onNavigationClicked: () -> Unit, onInfoClicked: () -> Unit) {
    TopAppBar(
        title = { Text(text = stringResource(R.string.manage_plates)) },
        navigationIcon = {
            IconButton(
                onClick = onNavigationClicked,
            ) {
                Icon(
                    imageVector = Icons.AutoMirrored.Filled.ArrowBack,
                    contentDescription = stringResource(R.string.go_back),
                    tint = MaterialTheme.colorScheme.primary
                )
            }
        },
        actions = {
            IconButton(onClick = onInfoClicked) {
                Icon(
                    imageVector = Icons.Default.Info,
                    contentDescription = stringResource(R.string.info),
                    tint = MaterialTheme.colorScheme.primary
                )
            }
        }
    )
}

@Composable
fun PlateCard(
    modifier: Modifier,
    plate: Plate,
    onDeleteClick: () -> Unit,
    onEditClick: () -> Unit
) {
    Card(
        modifier = modifier,
        elevation = CardDefaults.elevatedCardElevation(),
        colors = CardDefaults.elevatedCardColors(
            containerColor = MaterialTheme.colorScheme.surfaceColorAtElevation(2.dp)
        ),
        onClick = onEditClick
    ) {
        Row(
            modifier = Modifier.padding(8.dp)
        ) {
            if (plate.imageUri == null) {
                Icon(
                    painter = painterResource(id = R.drawable.baseline_image_not_supported_24),
                    contentDescription = null,
                    modifier = Modifier
                        .padding(start = 8.dp)
                        .padding(end = 16.dp)
                        .size(40.dp)
                        .align(Alignment.CenterVertically)
                        .alpha(0.6f)
                )
            } else {
                AsyncImage(
                    model = plate.imageUri,
                    contentScale = ContentScale.Crop,
                    contentDescription = null,
                    modifier = Modifier
                        .padding(start = 8.dp)
                        .padding(end = 16.dp)
                        .size(40.dp)
                        .align(Alignment.CenterVertically)
                        .clip(RoundedCornerShape(8.dp)),
                )
            }
            Text(
                text = "${plate.name}\n" + if (UnitLocale.default == UnitLocale.Metric)
                    stringResource(R.string.gram, plate.weight.toInt())
                else stringResource(R.string.ounce, plate.weight.toInt()),
                modifier = Modifier
                    .align(Alignment.CenterVertically)
                    .weight(1f)
            )
            IconButton(
                onClick = onEditClick,
                modifier = Modifier.align(Alignment.CenterVertically)
            ) {
                Icon(
                    imageVector = Icons.Filled.Edit,
                    contentDescription = stringResource(id = R.string.edit_plate),
                    tint = MaterialTheme.colorScheme.primary
                )
            }
            IconButton(
                onClick = onDeleteClick,
                modifier = Modifier.align(Alignment.CenterVertically)
            ) {
                Icon(
                    imageVector = Icons.Filled.Delete,
                    contentDescription = stringResource(R.string.delete_plate),
                    tint = MaterialTheme.colorScheme.primary
                )
            }
        }
    }
}