package com.portio.ui.screens.usermanager

import androidx.activity.compose.BackHandler
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.layout.wrapContentHeight
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.filled.ArrowDropDown
import androidx.compose.material.icons.filled.Done
import androidx.compose.material.icons.filled.Info
import androidx.compose.material.icons.outlined.Add
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.Button
import androidx.compose.material3.Card
import androidx.compose.material3.CardDefaults
import androidx.compose.material3.DropdownMenu
import androidx.compose.material3.DropdownMenuItem
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TextButton
import androidx.compose.material3.TextField
import androidx.compose.material3.TopAppBar
import androidx.compose.material3.surfaceColorAtElevation
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.alpha
import androidx.compose.ui.draw.clip
import androidx.compose.ui.draw.rotate
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.TextRange
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardCapitalization
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.input.TextFieldValue
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import coil.compose.AsyncImage
import com.portio.R
import com.portio.RatioMode
import com.portio.data.User
import java.net.URLDecoder
import java.nio.charset.StandardCharsets
import kotlin.math.min

@Composable
fun getRatioModeString(ratioMode: RatioMode): String {
    return when (ratioMode) {
        RatioMode.EQUALLY -> stringResource(R.string.equally)
        RatioMode.PARTS -> stringResource(R.string.parts)
        RatioMode.AMOUNT -> stringResource(R.string.amount)
    }
}

@Composable
fun UserManagerScreen(
    userList: List<User>,
    ratioMode: RatioMode,
    modifier: Modifier = Modifier,
    onNavigationClicked: () -> Unit,
    onUserRemoved: (User) -> Unit,
    onAddOrEditUser: (User) -> Unit,
    onChangeRatioMode: (RatioMode) -> Unit,
) {
    // Dialog to ask if user is really sure to exit.
    var showExitDialog by remember { mutableStateOf(false) }
    if (showExitDialog) {
        ExitDialog(onDismiss = { exit: Boolean ->
            showExitDialog = false
            if (exit) {
                onChangeRatioMode(RatioMode.EQUALLY)
                onNavigationClicked()
            }
        })
    }

    val isSumError = ratioMode == RatioMode.AMOUNT && userList.sumOf { it.effectiveRatio } != 100
    BackHandler {
        if (isSumError) showExitDialog = true
        else onNavigationClicked()
    }

    var deleteDialog by remember { mutableStateOf<User?>(null) }
    if (deleteDialog != null) {
        AlertDialog(
            onDismissRequest = { deleteDialog = null },
            title = { Text(text = stringResource(R.string.are_you_sure)) },
            text = { Text(text = stringResource(R.string.delete_user_section)) },
            dismissButton = {
                TextButton(onClick = { deleteDialog = null }) {
                    Text(text = stringResource(R.string.cancel))
                }
            },
            confirmButton = {
                TextButton(
                    onClick = {
                        deleteDialog?.let {
                            onUserRemoved(it)
                            deleteDialog = null
                        }
                    }) {
                    Text(text = stringResource(R.string.delete))
                }
            }
        )
    }

    var showInfoDialog by remember { mutableStateOf(false) }
    if (showInfoDialog) {
        AlertDialog(
            onDismissRequest = { showInfoDialog = false },
            title = { Text(text = stringResource(R.string.manage_users_info_title)) },
            text = { Text(text = stringResource(R.string.manage_users_description)) },
            confirmButton = {
                Button(onClick = { showInfoDialog = false }) {
                    Text(text = stringResource(android.R.string.ok))
                }
            }
        )
    }

    val addOrEditUserDialog = remember { mutableStateOf<Pair<Boolean, User?>>(Pair(false, null)) }
    if (addOrEditUserDialog.value.first) AddOrEditUserDialog(
        onAddOrEditUser = { user ->
            onAddOrEditUser(user)
            addOrEditUserDialog.value = Pair(false, null)
        },
        onDismiss = { addOrEditUserDialog.value = Pair(false, null) },
        user = addOrEditUserDialog.value.second,
        userList = userList
    )

    var menuExpanded by remember { mutableStateOf(false) }
    val options = listOf(RatioMode.EQUALLY, RatioMode.PARTS, RatioMode.AMOUNT)
    var expandedItem by remember { mutableStateOf<User?>(null) }
    Scaffold(modifier = modifier,
        topBar = {
            UserManagerScreenTopBar(onNavigationClicked = {
                if (isSumError) showExitDialog = true
                else onNavigationClicked()
            },
                onInfoClicked = { showInfoDialog = true })
        },
        floatingActionButton = {
            Button(
                onClick = { addOrEditUserDialog.value = Pair(true, null) }
            ) {
                Icon(
                    painter = painterResource(id = android.R.drawable.ic_input_add),
                    contentDescription = null
                )
                Text(
                    modifier = Modifier.padding(start = 4.dp),
                    text = stringResource(R.string.add_user_title)
                )
            }
        }
    ) { innerPadding ->
        Column(
            verticalArrangement = Arrangement.Center,
            modifier = Modifier
                .fillMaxWidth()
                .padding(innerPadding)
                .padding(bottom = 32.dp)
        ) {
            if (userList.isEmpty())
                Text(
                    text = stringResource(R.string.no_user),
                    modifier = Modifier
                        .fillMaxWidth()
                        .padding(horizontal = 12.dp),
                    style = MaterialTheme.typography.bodyLarge
                )
            else {
                Box(
                    modifier = Modifier
                        .padding(horizontal = 12.dp)
                        .align(Alignment.End)
                ) {
                    TextButton(
                        modifier = Modifier,
                        onClick = { menuExpanded = true }) {
                        Text(
                            text =
                            stringResource(R.string.ratio_mode, getRatioModeString(ratioMode))
                        )
                        Icon(
                            Icons.Filled.ArrowDropDown,
                            null,
                            modifier.rotate(if (menuExpanded) 180f else 0f)
                        )
                    }
                    DropdownMenu(
                        expanded = menuExpanded,
                        onDismissRequest = { menuExpanded = false },
                        modifier = Modifier,
                    ) {
                        options.forEach { option ->
                            DropdownMenuItem(
                                text = {
                                    Text(
                                        text = getRatioModeString(option),
                                        color = MaterialTheme.colorScheme.primary
                                    )
                                },
                                onClick = {
                                    onChangeRatioMode(option)
                                    menuExpanded = false
                                }
                            )
                        }
                    }
                }

                LazyColumn(
                    modifier = Modifier
                        .weight(1f)
                        .padding(bottom = 32.dp)
                        .padding(horizontal = 12.dp)
                ) {
                    items(userList.size) { index ->
                        Box {
                            UserCard(
                                modifier = Modifier
                                    .fillMaxWidth()
                                    .padding(vertical = 4.dp),
                                user = userList[index],
                                ratioMode = ratioMode,
                                onRatioModified = { ratio ->
                                    onAddOrEditUser(userList[index].copy(desiredRatio = ratio))
                                },
                                onClick = { expandedItem = userList[index] },
                                isError = isSumError
                            )
                            DropdownMenu(
                                expanded = expandedItem == userList[index],
                                onDismissRequest = { expandedItem = null }
                            ) {
                                DropdownMenuItem(
                                    text = {
                                        Text(
                                            text = stringResource(R.string.edit),
                                            color = MaterialTheme.colorScheme.primary
                                        )
                                    },
                                    onClick = {
                                        addOrEditUserDialog.value = Pair(true, userList[index])
                                        expandedItem = null
                                    })
                                DropdownMenuItem(
                                    text = {
                                        Text(
                                            text = stringResource(R.string.delete),
                                            color = MaterialTheme.colorScheme.primary
                                        )
                                    },
                                    onClick = {
                                        deleteDialog = userList[index]
                                        expandedItem = null
                                    })
                            }
                        }
                    }
                }
            }
        }
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun UserManagerScreenTopBar(onNavigationClicked: () -> Unit, onInfoClicked: () -> Unit) {
    TopAppBar(
        title = { Text(text = stringResource(R.string.manage_users)) },
        navigationIcon = {
            IconButton(
                onClick = onNavigationClicked,
            ) {
                Icon(
                    imageVector = Icons.AutoMirrored.Filled.ArrowBack,
                    contentDescription = stringResource(R.string.go_back),
                    tint = MaterialTheme.colorScheme.primary
                )
            }
        },
        actions = {
            IconButton(onClick = onInfoClicked) {
                Icon(
                    imageVector = Icons.Default.Info,
                    contentDescription = stringResource(R.string.info),
                    tint = MaterialTheme.colorScheme.primary
                )
            }
        })
}

@Composable
fun UserCard(
    user: User,
    ratioMode: RatioMode,
    onClick: () -> Unit,
    onRatioModified: (Int) -> Unit,
    modifier: Modifier = Modifier,
    isError: Boolean = false
) {
    Card(
        onClick = { onClick() },
        modifier = modifier,
        elevation = CardDefaults.elevatedCardElevation(),
        colors = CardDefaults.elevatedCardColors(
            containerColor = MaterialTheme.colorScheme.surfaceColorAtElevation(2.dp)
        )
    ) {
        Row(
            modifier = Modifier.padding(8.dp)
        ) {
            if (user.imageUri == null) {
                Icon(
                    painter = painterResource(id = R.drawable.baseline_image_not_supported_24),
                    contentDescription = null,
                    modifier = Modifier
                        .padding(start = 8.dp)
                        .padding(end = 16.dp)
                        .size(40.dp)
                        .align(Alignment.CenterVertically)
                        .alpha(0.6f)
                )
            } else {
                val decodedUri = URLDecoder
                    .decode(user.imageUri.toString(), StandardCharsets.UTF_8.toString())
                AsyncImage(
                    model = decodedUri,
                    contentDescription = null,
                    modifier = Modifier
                        .padding(start = 8.dp)
                        .padding(end = 16.dp)
                        .size(40.dp)
                        .align(Alignment.CenterVertically)
                        .clip(RoundedCornerShape(8.dp)),
                )
            }
            Text(
                text = user.name,
                modifier = Modifier
                    .align(Alignment.CenterVertically)
                    .weight(1f)
            )

            when (ratioMode) {
                RatioMode.EQUALLY -> Text(
                    text = stringResource(R.string.percentage, user.effectiveRatio),
                    modifier = Modifier
                        .align(Alignment.CenterVertically)
                        .padding(end = 8.dp)
                        .height(48.dp)
                        .wrapContentHeight()
                )

                RatioMode.PARTS -> PartSelector(
                    user.desiredRatio,
                    user.effectiveRatio,
                    onRatioModified
                )

                RatioMode.AMOUNT -> AmountSelector(
                    user.desiredRatio,
                    user.effectiveRatio,
                    onRatioModified,
                    isError
                )
            }
        }
    }
}


@Composable
fun PartSelector(desiredRatio: Int, effectiveRatio: Int, onRatioChanged: (Int) -> Unit) {
    Row(verticalAlignment = Alignment.CenterVertically) {
        Column(modifier = Modifier.padding(end = 4.dp), horizontalAlignment = Alignment.End) {
            Text(text = stringResource(R.string.part_ratio, desiredRatio))
            Text(
                text = stringResource(R.string.percentage, effectiveRatio),
                style = MaterialTheme.typography.bodyMedium.copy(
                    lineHeight = MaterialTheme.typography.bodyMedium.fontSize
                )
            )
        }
        IconButton(
            onClick = { onRatioChanged(desiredRatio - 1) },
            modifier = Modifier.align(Alignment.CenterVertically),
            enabled = desiredRatio > 0
        ) {
            Icon(
                painter = painterResource(id = R.drawable.baseline_remove_24),
                contentDescription = stringResource(R.string.minus),
                tint = MaterialTheme.colorScheme.primary
            )
        }
        IconButton(
            onClick = { onRatioChanged(desiredRatio + 1) },
            modifier = Modifier.align(Alignment.CenterVertically)
        ) {
            Icon(
                imageVector = Icons.Outlined.Add,
                contentDescription = stringResource(R.string.plus),
                tint = MaterialTheme.colorScheme.primary
            )
        }
    }
}

@Composable
fun AmountSelector(
    desiredRatio: Int,
    effectiveRatio: Int,
    onRatioChanged: (Int) -> Unit,
    isError: Boolean
) {
    val value = if (desiredRatio != -1) desiredRatio else effectiveRatio
    var textFieldSelection by remember {
        mutableStateOf(TextRange(value.toString().length))
    }
    TextField(
        modifier = Modifier
            .width(90.dp)
            .height(48.dp),
        textStyle = TextStyle(fontSize = 16.sp),
        value = run {
            if (value == 0) { // Force cursor to be at the end
                textFieldSelection = TextRange(1)
            }
            TextFieldValue(text = value.toString(), selection = textFieldSelection)
        },
        isError = isError,
        onValueChange = { textFieldValue ->
            val textFieldText = textFieldValue.text.filter { txt -> txt.isDigit() }.let {
                if (it.isEmpty()) 0 else it.toInt()
            }
            textFieldSelection = textFieldValue.selection
            onRatioChanged(
                min(textFieldText, 100)
            )
        },
        suffix = { Text(text = "%") },
        keyboardOptions = KeyboardOptions(
            keyboardType = KeyboardType.Number,
            imeAction = ImeAction.Done
        ),
    )
}

@Composable
fun AddOrEditUserDialog(
    onAddOrEditUser: (User) -> Unit,
    onDismiss: () -> Unit,
    user: User?,
    userList: List<User>,
    modifier: Modifier = Modifier
) {
    val isEditing = user != null
    var editedUser by remember { mutableStateOf(user ?: User.default()) }

    // Check if the user name is already in the list
    // (or if editing, if it is in the list but not the same user).
    val nameAlreadyExist = userList
        .firstOrNull { it.name == editedUser.name && it.id != editedUser.id } != null
    val isEmptyError = editedUser.name.isEmpty()
    val isTooLongError = editedUser.name.length > User.MAX_LENGTH
    val isNameError = isEmptyError || nameAlreadyExist || isTooLongError

    AlertDialog(
        modifier = modifier,
        onDismissRequest = onDismiss,
        title = {
            Text(
                text = if (isEditing) stringResource(R.string.edit_user_title)
                else stringResource(R.string.add_user_title)
            )
        },
        text = {
            TextField(
                keyboardOptions = KeyboardOptions(
                    capitalization = KeyboardCapitalization.Words,
                    keyboardType = KeyboardType.Text,
                    imeAction = ImeAction.Done
                ),
                supportingText = {
                    Row {
                        Text(
                            if (nameAlreadyExist) stringResource(R.string.user_already_exists)
                            else if (isTooLongError) stringResource(R.string.user_name_too_long)
                            else if (isEmptyError) stringResource(R.string.user_name_empty)
                            else "",
                        )
                        Spacer(Modifier.weight(1f))
                        Text(
                            stringResource(
                                R.string.size_limit,
                                editedUser.name.length,
                                User.MAX_LENGTH
                            )
                        )
                    }
                },
                isError = isNameError,
                value = editedUser.name,
                onValueChange = { editedUser = editedUser.copy(name = it) },
                label = { Text(text = stringResource(R.string.user_name)) },
                modifier = Modifier.fillMaxWidth()
            )
        },
        dismissButton = {
            TextButton(onClick = onDismiss) {
                Text(stringResource(R.string.cancel))
            }
        },
        confirmButton = {
            Button(
                enabled = !isNameError,
                onClick = { onAddOrEditUser(editedUser) },
            ) {
                Icon(
                    imageVector = Icons.Filled.Done,
                    contentDescription = null,
                )
                Text(
                    modifier = Modifier.padding(start = 4.dp),
                    text = stringResource(R.string.done)
                )
            }
        }
    )
}

@Composable
fun ExitDialog(onDismiss: (Boolean) -> Unit) {
    AlertDialog(
        onDismissRequest = { onDismiss(false) },
        title = { Text(text = stringResource(R.string.are_you_sure)) },
        text = { Text(text = stringResource(R.string.invalid_amount_setting)) },
        dismissButton = {
            TextButton(onClick = { onDismiss(false) }) {
                Text(stringResource(R.string.cancel))
            }
        },
        confirmButton = {
            TextButton(onClick = { onDismiss(true) }) {
                Text(stringResource(R.string.exit))
            }
        }
    )
}