package com.portio.utils

import android.content.Context
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.Matrix
import androidx.exifinterface.media.ExifInterface
import android.net.Uri
import androidx.core.content.FileProvider
import androidx.core.util.TypedValueCompat.dpToPx
import java.io.File
import java.io.FileOutputStream
import java.util.UUID
import kotlin.math.min

// Crop to have a square image
// Resize to respect maxDp
// Rotate to the correct orientation
fun cropResizeRotateAndSaveImage(
    context: Context,
    uri: Uri,
    newFile: File,
    maxDp: Float,
): Boolean {

    val maxPx = dpToPx(maxDp, context.resources.displayMetrics).toInt()
    val contentResolver = context.contentResolver

    // Get the rotation angle of the image
    val rotationDegrees = try {
        contentResolver.openInputStream(uri)?.use { inputStream ->
            val exif = ExifInterface(inputStream)
            when (exif.getAttributeInt(
                ExifInterface.TAG_ORIENTATION,
                ExifInterface.ORIENTATION_NORMAL
            )) {
                ExifInterface.ORIENTATION_ROTATE_90 -> 90f
                ExifInterface.ORIENTATION_ROTATE_180 -> 180f
                ExifInterface.ORIENTATION_ROTATE_270 -> 270f
                else -> 0f
            }
        } ?: 0f
    } catch (e: Exception) {
        0f
    }

    // Load the original bitmap
    val inputStream = contentResolver.openInputStream(uri) ?: return false
    val originalBitmap = BitmapFactory.decodeStream(inputStream) ?: return false

    val originalWidth = originalBitmap.width
    val originalHeight = originalBitmap.height

    // Define the size of the square to crop
    val size = min(originalWidth, originalHeight)

    // Get the offset to crop the image
    val xOffset = (originalWidth - size) / 2 // Centering horizontally
    val yOffset = (originalHeight - size) / 2 // Centering vertically

    // Crop the image
    val croppedBitmap = Bitmap.createBitmap(originalBitmap, xOffset, yOffset, size, size)

    // Resize the image
    val resizedBitmap = Bitmap.createScaledBitmap(croppedBitmap, maxPx, maxPx, true)

    // Apply rotation using Matrix
    val matrix = Matrix()
    matrix.postRotate(rotationDegrees) // Apply the specified rotation angle
    val rotatedBitmap = Bitmap.createBitmap(
        resizedBitmap,
        0,
        0,
        resizedBitmap.width,
        resizedBitmap.height,
        matrix,
        true
    )

    // Save the image to the file
    FileOutputStream(newFile).use { outputStream ->
        rotatedBitmap.compress(Bitmap.CompressFormat.PNG, 80, outputStream)
    }

    return true
}

fun createImageFile(context: Context, plateId: UUID): Uri {
    val file = File
        .createTempFile("temp_plate_image_${plateId}", ".jpg", context.cacheDir)
    return FileProvider.getUriForFile(context, "${context.packageName}.fileprovider", file)
}