package com.portio.utils

import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.Canvas
import android.util.Log
import java.io.InputStream
import java.net.HttpURLConnection
import java.net.URL

object UserAvatarUtils {

    private val TAG = UserAvatarUtils::class.simpleName

    fun processUserAvatar(username: String): Bitmap? {
        val avatarBaseBitmap = downloadRoboHashImage(username) ?: return null

        val contrastedColor = getContrastedColor(getDominantColor(avatarBaseBitmap))
        val outputBitmap = combineImageWithBackground(avatarBaseBitmap, contrastedColor)

        return outputBitmap
    }

    /**
     * Downloads a RoboHash image for the given username and returns it as a Bitmap.
     *
     * @param username The username to generate the RoboHash image.
     * @return The Bitmap of the downloaded image, or null if an error occurred.
     */
    private fun downloadRoboHashImage(username: String): Bitmap? {
        // Construct the URL for the RoboHash image
        val imageUrl = "https://robohash.org/$username.png"

        return try {
            // Create a URL object
            val url = URL(imageUrl)

            // Open a connection to the URL
            val connection = url.openConnection() as HttpURLConnection
            connection.requestMethod = "GET" // Set the request method to GET
            connection.connect() // Initiate the connection

            // Check if the response code indicates success
            if (connection.responseCode == HttpURLConnection.HTTP_OK) {
                // Retrieve the input stream from the connection
                val inputStream: InputStream = connection.inputStream
                val bitmap =
                    BitmapFactory.decodeStream(inputStream) // Decode the stream into a Bitmap
                inputStream.close() // Close the input stream
                Log.i(TAG, "Image downloaded successfully")
                bitmap
            } else {
                // Handle HTTP errors
                Log.e(TAG, "Failed to download image. HTTP Response Code: ${connection.responseCode}")
                null
            }.also {
                // Ensure the connection is closed
                connection.disconnect()
            }
        } catch (e: Exception) {
            // Log and handle any exceptions
            Log.e(TAG, "Error downloading image", e)
            null
        }
    }

    /**
     * Adjusts the saturation of the given RGB color to create a contrasting color.
     *
     * @param rgbColor A Triple containing components of the color (each between 0 and 255).
     * @return A Triple containing the red, green, and blue components of the adjusted color.
     */
    private fun getContrastedColor(rgbColor: Triple<Int, Int, Int>): Triple<Int, Int, Int> {
        // Convert the input RGB color to HSV format
        val hsv = rgbToHsv(rgbColor.first, rgbColor.second, rgbColor.third)

        // Extract hue, saturation, and value components from the HSV color
        println("Hue: ${hsv.first}, Saturation: ${hsv.second}, Value: ${hsv.third}")
        val hue = hsv.first
        var saturation = hsv.second
        val value = hsv.third

        // Adjust the saturation: increase it by 0.5 if it's low, or decrease it by 0.5 if it's high
        if (saturation >= 0.5) saturation -= 0.50
        else saturation += 0.50

        // Convert the modified HSV color back to RGB format
        val adjustedRgb = hsvToRgb(hue, saturation, value)

        // Return the adjusted RGB color
        return adjustedRgb
    }

    /**
     * Finds the dominant (most frequent) color in a given Bitmap image.
     *
     * @param image The Bitmap image to analyze.
     * @return A Triple containing the Red, Green, and Blue components (0-255) of the dominant color.
     */
    private fun getDominantColor(image: Bitmap): Triple<Int, Int, Int> {
        // Map to store the count of each color in the image
        val colorFrequency = mutableMapOf<Int, Int>()

        // Iterate over each pixel in the image
        for (x in 0 until image.width) {
            for (y in 0 until image.height) {
                val pixel = image.getPixel(x, y) // Get the pixel color
                val alpha = android.graphics.Color.alpha(pixel) // Extract the alpha channel

                // Ignore fully transparent pixels
                if (alpha > 0) {
                    val rgbColor = pixel and 0xFFFFFF // Remove the alpha channel
                    colorFrequency[rgbColor] = colorFrequency.getOrDefault(rgbColor, 0) + 1
                }
            }
        }

        // Find the color with the highest frequency
        val dominantColor = colorFrequency.maxByOrNull { it.value }?.key ?: 0xFFFFFF

        // Extract the RGB components from the dominant color
        val red = (dominantColor shr 16) and 0xFF
        val green = (dominantColor shr 8) and 0xFF
        val blue = dominantColor and 0xFF

        // Return the dominant color as an RGB Triple
        return Triple(red, green, blue)
    }

    /**
     * Converts an RGB color to HSV (Hue, Saturation, Value) color space.
     *
     * @param r Red component (0-255)
     * @param g Green component (0-255)
     * @param b Blue component (0-255)
     * @return A Triple containing Hue (0-360 degrees), Saturation (0.0-1.0), and Value (0.0-1.0).
     */
    private fun rgbToHsv(r: Int, g: Int, b: Int): Triple<Double, Double, Double> {
        // Step 1: Normalize RGB values to the range [0, 1]
        val redNormalized = r / 255.0
        val greenNormalized = g / 255.0
        val blueNormalized = b / 255.0

        // Step 2: Compute maximum, minimum, and delta values
        val maxColor = maxOf(redNormalized, greenNormalized, blueNormalized)
        val minColor = minOf(redNormalized, greenNormalized, blueNormalized)
        val delta = maxColor - minColor

        // Step 3: Calculate Hue
        val hue = when {
            delta == 0.0 -> 0.0 // No color dominance, hue is undefined
            maxColor == redNormalized -> 60 * ((greenNormalized - blueNormalized) / delta % 6)
            maxColor == greenNormalized -> 60 * ((blueNormalized - redNormalized) / delta + 2)
            else -> 60 * ((redNormalized - greenNormalized) / delta + 4)
        }.let { if (it < 0) it + 360 else it } // Ensure positive hue

        // Step 4: Calculate Saturation
        val saturation = if (maxColor == 0.0) 0.0 else delta / maxColor

        // Step 5: Calculate Value
        val value = maxColor

        return Triple(hue, saturation, value)
    }

    /**
     * Converts an HSV color to RGB (Red, Green, Blue) color space.
     *
     * @param h Hue (0-360 degrees)
     * @param s Saturation (0.0-1.0)
     * @param v Value (0.0-1.0)
     * @return A Triple containing Red, Green, and Blue components (0-255).
     */
    private fun hsvToRgb(h: Double, s: Double, v: Double): Triple<Int, Int, Int> {
        // Step 1: Calculate chroma (C), secondary color (X), and match value (M)
        val chroma = v * s
        val secondaryColor = chroma * (1 - kotlin.math.abs((h / 60) % 2 - 1))
        val matchValue = v - chroma

        // Step 2: Determine the primary RGB values based on the hue segment
        val (redPrime, greenPrime, bluePrime) = when (h) {
            in 0.0..60.0 -> Triple(chroma, secondaryColor, 0.0)
            in 60.0..120.0 -> Triple(secondaryColor, chroma, 0.0)
            in 120.0..180.0 -> Triple(0.0, chroma, secondaryColor)
            in 180.0..240.0 -> Triple(0.0, secondaryColor, chroma)
            in 240.0..300.0 -> Triple(secondaryColor, 0.0, chroma)
            else -> Triple(chroma, 0.0, secondaryColor) // 300.0..360.0
        }

        // Step 3: Adjust the RGB values to fit the [0, 255] range
        val red = ((redPrime + matchValue) * 255).toInt()
        val green = ((greenPrime + matchValue) * 255).toInt()
        val blue = ((bluePrime + matchValue) * 255).toInt()

        return Triple(red, green, blue)
    }

    /**
     * Combines an overlay image with a solid background color into a single Bitmap.
     *
     * @param overlayBitmap The overlay image to be drawn on top.
     * @param backgroundColor The RGB color for the background represented as a Triple (Red, Green, Blue).
     * @return A Bitmap with the overlay image drawn on top of the solid color background.
     */
    private fun combineImageWithBackground(
        overlayBitmap: Bitmap,
        backgroundColor: Triple<Int, Int, Int>
    ): Bitmap {
        // Create a solid color background Bitmap matching the size of the overlay
        val backgroundBitmap = generateSolidColorBitmap(
            overlayBitmap.width,
            overlayBitmap.height,
            backgroundColor
        )

        // Create a new Bitmap to combine the background and overlay
        val resultBitmap =
            Bitmap.createBitmap(overlayBitmap.width, overlayBitmap.height, Bitmap.Config.ARGB_8888)
        val canvas = Canvas(resultBitmap)

        // Draw the background first
        canvas.drawBitmap(backgroundBitmap, 0f, 0f, null)

        // Draw the overlay image on top
        canvas.drawBitmap(overlayBitmap, 0f, 0f, null)

        return resultBitmap
    }

    /**
     * Creates a solid color Bitmap with the specified width, height, and RGB color.
     *
     * @param width The width of the image.
     * @param height The height of the image.
     * @param rgbColor The RGB color represented as a Triple (Red, Green, Blue).
     * @return A Bitmap filled with the specified color.
     */
    private fun generateSolidColorBitmap(
        width: Int,
        height: Int,
        rgbColor: Triple<Int, Int, Int>
    ): Bitmap {
        val (red, green, blue) = rgbColor
        // Create a blank Bitmap with ARGB_8888 configuration
        val bitmap = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888)
        // Draw the solid color onto the Bitmap
        val canvas = Canvas(bitmap)
        canvas.drawColor(android.graphics.Color.argb(50, red, green, blue))
        return bitmap
    }
}
